function [expandersList] = expandAllRegions(srcHStack, dstHStack,  srcImg, dstImg, doCornerTrick, numberOfFurtherExpansions)
% main function for phase ONE points correspondences
% 
% input:    srcHStack, dstHStack - source and destination normalizing
%                                  matrices
%           srcImg, dstImg - source and destination images
%           soCornerTrick - logic value for parameter activation
% 
% output:   expanderList - arrayList (see arrayList function) of expanders
% 
% 

            %%%%DONE%%%%
%for debug purposes choose specific indexes:
focusOnIdxs =1:size(srcHStack, 3);
srcHStack = srcHStack(:, :, focusOnIdxs);dstHStack = dstHStack(:, :, focusOnIdxs);
% prealocation
expanders = regionExpanderDeltaBased([1, size(srcHStack,3)]); 
% initializing expanders object
for i = 1:size(srcHStack, 3)
    expanders(i) = regionExpanderDeltaBased(srcHStack(:,:,i), dstHStack(:,:,i),i);
end
% corner computation over source image for texture aware phase-one sampling
if (doCornerTrick)
    srcCornerImg = cornermetric(srcImg, 'MinimumEigenvalue', 'FilterCoefficients', fspecial('average', [(expanders(1).templateExtent*2+1) 1]));
else
    srcCornerImg = [];
end

% expansion
expanstionRatio = [1.5 2*ones(1,numberOfFurtherExpansions)];

% Uncomment for parallelization
% if(isempty(gcp('nocreate')))
%    parpool 
% end
% if(~isempty(gcp('nocreate')))
%    pctRunOnAll('warning','off')
% end
%parfor expandersIdx = 1:length(expanders)  %outer loop - independent iterations over all expanders
for expandersIdx = 1:length(expanders)  %outer loop - independent iterations over all expanders
    expander = expanders(expandersIdx);
    for expIteration = 1:length(expanstionRatio)    %inner loop - DEPENDENT iterations over all expansions (for each expander)
        if (~((expander.wasStopped) || (expander.wasRejected)))
            expander.expand(srcImg, dstImg, srcCornerImg, expanstionRatio(expIteration), false);          
        else
            break
        end
    end
    expanders(expandersIdx) = expander;
end
expandersList = ArrayList(expanders); %arrayList casting of expanders for comfort reasons (see arrayList.m)
end

