function [ reProjErrs, mahalanobisErrs ] = evalWeightedAffineEstimation( Hhat, transformationErrCovs, srcPts, tgtPts, errCovs )
%evalWeightedAffineEstimation Evaluate the agrrement of the given matches
%with a given affine transformation
%   Detailed explanation goes here
%   Inputs:
%   Hhat - the estimated affine transform
%   transformationErrCovs - The uncerteinty covariance of predicting the
%   locations of the target points using the given estimation (Typically
%   given by weighteAffineEstimation)
%   srcPts - points in the first image
%   tgtPts - corresponding points in the target image
%   errCovs - the uncerteinty covariance matrices of each
%   match

%   Outputs:
%   reProjErrs - The reprojection errors of Hhat on the given matches
%   mahalanobisErrs - The mahalanobis error of Hhat on the given matches -
%   Since we have two types of covariances, we take the minimum
%   (optimistic) mahalonobis error from each reprojection error.

reProjErrs = (tgtPts - doAffineTransform(srcPts,Hhat));
transformationMahalanobis = arrayfun(@(x) reProjErrs(x,:)/transformationErrCovs(:,:,x)*reProjErrs(x,:)',1:size(reProjErrs,1));
correlationMahalanobis = arrayfun(@(x) reProjErrs(x,:)/errCovs(:,:,x)*reProjErrs(x,:)',1:size(reProjErrs,1));
mahalanobisErrs = min(correlationMahalanobis,transformationMahalanobis);

end


%% debug
% eignVals = sqrt(reshape(diag(errorEignVals),2,length(diag(errorEignVals))/2)');
% imshowAndScatter(I1,srcPts,[],[],[],true);
% imshowAndScatter(I2,tgtPts,[],[],[],true);
% for k = 1:size(tgtPts,1)
%     h = error_ellipse(errCovs(:,:,k), tgtPts(k,:))  ;set(h, 'color','m','LineWidth',3);
%     line([tgtPts(k,1) (tgtPts(k,1) + eignVals(k,1)*errorEignVecs(1,1,k)')], [tgtPts(k,2) (tgtPts(k,2) + eignVals(k,1)*errorEignVecs(2,1,k)')],'color','b','LineWidth',3)
%     line([tgtPts(k,1) (tgtPts(k,1) + eignVals(k,2)*errorEignVecs(1,2,k)')], [tgtPts(k,2) (tgtPts(k,2) + eignVals(k,2)*errorEignVecs(2,2,k)')],'color','b','LineWidth',3)
%     line([tgtPts(k,1) (tgtPts(k,1) + reProjErrs(k,1))], [tgtPts(k,2) (tgtPts(k,2) + reProjErrs(k,2))],'color','r','LineWidth',3)
% 
% end