function [Hhat,invHerrCovariance,predictabilityCovs,invCovMat] = weighteAffineEstimation( srcPts,tgtPts,invErrCovs, tstSrcPts)
%weighteAffineEstimation Estimate affine transformation from
%correspondences with uncerteinties
%   Inputs:
%   srcPts - points in the first image
%   tgtPts - corresponding points in the target image
%   invErrCovs - the inverse matrices of the uncerteinty covariance of each
%   match
%   tstSrcPts - optional input to asses the predictabillity of the given
%   points using the new estimated affine transformation

%   Outputs:
%   Hhat - the estimated affine transform
%   invHerrCovariance - Estimation of the inverse of the error covariance
%   of the affine estimation
%   predictabilityCovs - The estimated err covariance when estimating the
%   target positions of tstSrcPts
%   invCovMat - The full assembeled matrix of the given match uncertenties.
%   Delievered for use in other functions

Y = (reshape(tgtPts',1,[]))';
X = zeros(length(Y),6);
invCovMat = zeros(length(Y));
for k = 1:size(srcPts,1)
    invCovMat((2*k-1):2*k,(2*k-1):2*k) = invErrCovs(:,:,k);
end
X(1:2:end,1:3) = [srcPts ones(size(srcPts,1),1)];
X(2:2:end,4:6) = [srcPts ones(size(srcPts,1),1)];
invHerrCovariance = X'*invCovMat*X;
Hhat = [reshape(invHerrCovariance\(X'*invCovMat)*Y,3,2)'; 0  0 1];
if (nargin > 3 && nargout > 2)
    XT = zeros(2*size(tstSrcPts,1),6);
    XT(1:2:end,1:3) = [tstSrcPts ones(size(tstSrcPts,1),1)];
    XT(2:2:end,4:6) = [tstSrcPts ones(size(tstSrcPts,1),1)];
    predictabilityCovs = zeros(2,2,size(tstSrcPts,1));    

    for k = 1:size(tstSrcPts,1)                                             
        predictabilityCovs(:,:,k) = ((XT((2*k-1):2*k,:))/invHerrCovariance)*(XT((2*k-1):2*k,:)');
    end
end


    
end

