function [ verifiedSrcPoints, verifiedDstPoints, uncertaintyLevels, groupNumbers] = expandersListToPointMatches( expandersList )
% expandersListToPointMatches - extract the phase-one matches from all verified (Expanded) regions
% Inputs:
    % expandersList - A list of expanders containg the expanded regions
% Outputs:
    % verifiedSrcPoints - All the extracted points in the source image
    % verifiedDstPoints - All the corresponding points in the destination image
    % uncertaintyLevels - Uncertainty Levels for each match based on statistical measures
    % groupNumbers - assigning each verified match to the index (ID) of the region - for debug
verifiedSrcPoints = [];verifiedDstPoints = [];uncertaintyLevels = [];
k = 1;
for expandersIdx = 1:(expandersList.getLength)
    expander = expandersList.getMember(expandersIdx);
    if (expander.wasExpanded)
        verifiedSrcPoints = [verifiedSrcPoints;expander.currentCorrespondences.src];
        verifiedDstPoints = [verifiedDstPoints;expander.currentCorrespondences.dst];
        X = zeros(2*size(expander.currentCorrespondences.src,1),6);
        X(1:2:end,1:3) = [expander.currentCorrespondences.src ones(size(expander.currentCorrespondences.src,1),1)];
        X(2:2:end,4:6) = [expander.currentCorrespondences.src ones(size(expander.currentCorrespondences.src,1),1)];
        currentUncertaintyLevels = sqrt(arrayfun(@(k) det(X((2*k-1):(2*k),:) / expander.invHerrCovariance *(X((2*k-1):(2*k),:)')),1:size(expander.currentCorrespondences.src,1))) ./ expander.currentCorrespondences.qualities';
        uncertaintyLevels = [uncertaintyLevels  currentUncertaintyLevels];
        groupNumbers(k:(k + size(expander.currentCorrespondences.src,1) - 1)) = expandersIdx;
        k = k + size(expander.currentCorrespondences.src,1);
    end
    
end
verifiedDstPoints = double(verifiedDstPoints);
verifiedSrcPoints = double(verifiedSrcPoints);
end
