function [dstRois, dstPoints, badRois] = getTemplates(I, srcPoints, srcRoiExtent, T)
% crops a template of [srcRoiExtent] around srcPoints from I and transform
% it according to T (usually the transform between source and destination)
% 
% input :           I               - image from which we crop
%                   srcPoints       - set of points in I
%                   srcRoiExtent    - size of templates
%                   T               - affine transform matrix 
% 
% output:           dstRois         - templates
%                   dstPoints       - centeroids for templates
%                   bdRois          - templates conataining NAN (due to
%                                     inrepolation
% 
                    %%%%DONE%%%%
[zeroXpts,zeroYpts] = meshgrid(-srcRoiExtent:1:srcRoiExtent, -srcRoiExtent:1:srcRoiExtent);
dstPoints = doAffineTransform(srcPoints,T);
dstX = repmat(zeroXpts, 1, size(dstPoints,1)) + reshape((ones(size(zeroXpts,1)*size(zeroXpts,2),1)*(dstPoints(:,1)')),[size(zeroXpts,1) size(zeroXpts,2)*size(dstPoints,1)]);
dstY = repmat(zeroYpts, 1, size(dstPoints,1)) + reshape((ones(size(zeroYpts,1)*size(zeroYpts,1),1)*(dstPoints(:,2)')),[size(zeroYpts,1) size(zeroYpts,2)*size(dstPoints,1)]);
[Xq, Yq] = affineTransformGrid(dstX, dstY, invert2dAffine(T));
minXq = min(Xq(:)); maxXq = max(Xq(:)); minYq = min(Yq(:));maxYq = max(Yq(:)); 
minY = floor(minYq); minX = floor(minXq); maxX = ceil(maxXq);maxY = ceil(maxYq);
minY = max([minY 1]); minX = max([minX 1]); maxX = min([maxX size(I,2)]); maxY = min([maxY size(I,1)]);
interpedData = interp2((minX:maxX), (minY:maxY), double(I(minY:maxY,minX:maxX)), Xq, Yq);
dstRois = mat2cell(interpedData, size(zeroYpts,1), size(zeroXpts,2)*ones(size(dstPoints,1), 1));
badRois = any((reshape((any(isnan(interpedData))), (2*srcRoiExtent) + 1, size(srcPoints,1))), 1);           %roi idxs with nan-values
end


