function [miu,v1,v2,D,p] = getRegionPrincipleAxes(pointSet)
% getRegionPrincipleAxes get the principle axes of the ellipse fitted to
% the given points
% Inputs:
% pointSet - a set of 2D points
% Outputs:
% miu - The centroid of the set
% v1 - The major axis
% v2 - The minor axis
% D - The "square" of the second moment matrix
% p - An indicator to if the second moment matrix is positive definite...
...(p = 0 is healthy)

miu = mean(pointSet,1);
regionSize = size(pointSet,1);
centeredPixels = double(pointSet) - repmat(miu,regionSize,1);
M = (1/regionSize)*(centeredPixels')*centeredPixels;
[D,p] = chol(M);D = 2*D;
if (p == 0)
    [~,S,V] = svd(D);
    v1=S(1,1)*V(:,1);
    v2=S(2,2)*V(:,2);
else
    v1=[0;0];
    v2=[0;0];
end

