function [srcPatchInfo, dstPatchInfo, matches, scores, srcHarrisAffine, dstHarrisAffine] = getAffineAdaptedMatches(srcImg, dstImg, secondBestFactor)
% Phase Zero matching
% using Harris affine features + SIFT descriptor
% description.
% 
% input:        srcImg, dstImg - source and destination images
%               secondBestFactor- Throw match if  it is not at least
%               "secondBestFactor" better than the next best match.
%               
% 
% output:       srcPatchInfo, dstPatchInfo - source and destination vectors
%               of patchInfo structures containing the following fields:
%                           N - normalizing matrix (transformation to
%                               canonial pose)
%                           descriptor - binary descriptor values
%                           location -  center of the region


% feature detection and description
[frames, descriptors] = vl_covdet(im2single(srcImg),'estimateAffineShape', true,'estimateOrientation', true);
srcPatchInfo(length(frames)) = struct('N', [], 'descriptor', [], 'location', []);
for k = 1:size(descriptors,2)
    srcPatchInfo(k).descriptor = descriptors(:,k);
    srcPatchInfo(k).location = frames(:,k);
    srcPatchInfo(k).N =  vlFrameToAffineMatrix(frames(:,k));
end

[frames, descriptors] = vl_covdet(im2single(dstImg),'estimateAffineShape', true,'estimateOrientation', true);
dstPatchInfo(length(frames)) = struct('N', [], 'descriptor', [], 'location', []);
for k = 1:size(descriptors,2)
    dstPatchInfo(k).descriptor = descriptors(:,k);
    dstPatchInfo(k).location = frames(:,k);
    dstPatchInfo(k).N =  vlFrameToAffineMatrix(frames(:,k));
end

% descriptors comparison using VL_feat's comparator
[matches, scores] = vl_ubcmatch([srcPatchInfo.descriptor], [dstPatchInfo.descriptor], secondBestFactor) ;
srcLocs = [srcPatchInfo.location]';  srcLocs = srcLocs(:,1:2);
dstLocs = [dstPatchInfo.location]';  dstLocs = dstLocs(:,1:2);
srcHarrisAffine = srcLocs(matches(1,:),:);
dstHarrisAffine = dstLocs(matches(2,:),:);

end





