function [patchInfo] = getAffineInvariantDescriptorsAndCovariantFeatures(I, msers)
% Control function for binary invariant and covariant feature descriptor

widenRatio = 1.2;
NumOfSamples = 51;
patchInfo(length(msers)) = struct('N', [], 'descriptor', [], 'location', []);
[bandIndicators3, combinationFactors3] = buildRotationInvariantBandIdxs([NumOfSamples NumOfSamples], 3);
[bandIndicators4, combinationFactors4] = buildRotationInvariantBandIdxs([NumOfSamples NumOfSamples], 4);
invariantBandIndicators = cat(3, bandIndicators3, bandIndicators4);
invariantCombFactors = [[combinationFactors3 zeros(3,4)] ; [zeros(7,3) combinationFactors4]];
[covariantBandIndicators, covariantCombFactors] = buildRotationCovariantBandIdxs([NumOfSamples NumOfSamples], 8);
[xGrid, yGrid] = meshgrid(widenRatio*linspace(-1, 1, NumOfSamples), widenRatio*linspace(-1, 1, NumOfSamples));
extent = floor(NumOfSamples/2);
[xx, yy] = meshgrid(1:NumOfSamples,1:NumOfSamples);
[thetaCords, rCords] = cart2pol(xx - (extent+1),yy - (extent+1));
mask = rCords <= extent;requiredSamples = [xx(mask) yy(mask)];
rCords = rCords/extent;
allLocations = [msers.Location];
allAxes = [msers.Axes];
allOrientations = msers.Orientation;
% Uncomment for paralellization
% parfor i = 1:length(msers)
for i = 1:length(msers)
    location = allLocations(i,:);
    axes = allAxes(i,:);
    majorDirection = [cos(allOrientations(i)) -sin(allOrientations(i))];
    v1 = axes(1)*[majorDirection(1) majorDirection(2)];%*0.5 removed to get bigger regions
    v2 = axes(2)*[-majorDirection(2) majorDirection(1)];%*0.5 removed to get bigger regions
    Hn = [v1(1) v2(1) location(1); v1(2) v2(2) location(2); 0 0 1];
    [patchInfo(i).N, patchInfo(i).descriptor] = getAffineInvDescAndCovFeatureUsingBinaryTemplates(I, Hn, invariantBandIndicators, invariantCombFactors, covariantBandIndicators, covariantCombFactors, xGrid, yGrid, mask, requiredSamples, thetaCords, rCords);
    patchInfo(i).location = (location');
end

end

% profile on
% [ invariantBands3 ] = buildRotationInvariantBands( [51 51], 3 );
% [ invariantBands4 ] = buildRotationInvariantBands( [51 51], 4 );
% invariantBands = cat(3,invariantBands3,invariantBands4);
% [ covariantBands ] = buildRotationCovariantBands( [51 51], 8 );
% for i = 1:length(regions)
%     [ patchInfo(i).N, patchInfo(i).descriptor ] = getAffineInvDescsAndCovFeatureUsingGeneralInvCovBands( I, regions{i}, invariantBands, covariantBands);
%     patchInfo(i).feature = (mean(regions{i},1))';
% end
% profile off; profile viewer







