function [srcPatchInfo, dstPatchInfo, matches, scores, srcMSER, dstMSER] = getMSERMatches(srcImg, dstImg, secondBestFactor)
% Phase Zero matching
% using MSER feature detection, then Affine Invariant and Covariant feature
% description.
% 
% input:        srcImg, dstImg - source and destination images
%               secondBestFactor- Throw match if  it is not at least
%               "secondBestFactor" better than the next best match.
%               
% 
% output:       srcPatchInfo, dstPatchInfo - source and destination vectors
%               of patchInfo structures containing the following fields:
%                           N - normalizing matrix (transformation to
%                               canonial pose)
%                           descriptor - binary descriptor values
%                           location -  center of the region
% 
% 


                %%%%DONE%%%%
                
% MSER detection
srcMSERs = detectMSERFeatures(srcImg, 'RegionAreaRange', [36 10000]);
dstMSERs = detectMSERFeatures(dstImg, 'RegionAreaRange', [36 10000]);
matches = [];
scores = [];
srcMSER = [];
dstMSER = [];

if nargin < 5
  srcPatchInfo = [];  
end

% binary description of the regions detected by MSER
dstPatchInfo = [];
if (isempty(srcMSERs) || isempty(dstMSERs))
   return 
end

if isempty(srcPatchInfo)
    [srcPatchInfo] = getAffineInvariantDescriptorsAndCovariantFeatures(srcImg, srcMSERs);
end
[dstPatchInfo] = getAffineInvariantDescriptorsAndCovariantFeatures(dstImg, dstMSERs);

% descriptors comparison using VL_feat's comparator
[matches, scores] = vl_ubcmatch([srcPatchInfo.descriptor], [dstPatchInfo.descriptor], secondBestFactor) ;
srcLocs = [srcPatchInfo.location]';  srcLocs = srcLocs(:,1:2);
dstLocs = [dstPatchInfo.location]';  dstLocs = dstLocs(:,1:2);
srcMSER = srcLocs(matches(1,:),:);
dstMSER = dstLocs(matches(2,:),:);

end

